==> effector-reeffect-1.0.0.tgz: package/dist-src/runner.js <==
import { createNode, launch, step } from 'effector';
import { CancelledError, LimitExceededError } from './error';
import { QUEUE, RACE, TAKE_FIRST, TAKE_LAST } from './strategy';
import { cancellable } from './promise';
import { assign } from './tools';
export const patchRunner = (runner, scope) => {
    assign(runner.scope, scope, {
        anyway: runner.scope.anyway.prepend(payload => assign({ status: payload.result ? 'done' : 'fail' }, payload)),
    });
    runner.meta.onCopy.push('cancelled', 'cancel');
    runner.seq = seq();
    scope.cancel.watch(() => scope.cancelAll());
};
const seq = () => [
    step.run({
        fn({ params, args, req }, { getHandler, done, fail, anyway, cancelled, strategy, feedback, limit, running, push, unpush, cancelAll, }) {
            strategy = (args && args.strategy) || strategy;
            const scope = { params, strategy, feedback, push, unpush, cancelAll };
            const go = run(scope, getHandler(), fin(scope, [anyway, done], 'result', req.rs), fin(scope, [anyway, fail], 'error', req.rj), fin(scope, [cancelled], 'error', req.rj));
            if (running.length >= limit) {
                return go(new LimitExceededError(limit, running.length));
            }
            if (strategy === TAKE_FIRST && running.length > 0) {
                return go(new CancelledError(strategy));
            }
            if (strategy === TAKE_LAST && running.length > 0) {
                cancelAll(strategy);
            }
            if (strategy === QUEUE && running.length > 0) {
                const promise = cancellable(Promise.all(running.map(p => p.catch(() => { }))));
                push(promise);
                return promise.then(() => (unpush(promise), go()), error => (unpush(promise), go(error)));
            }
            go();
        },
    }),
];
const run = ({ params, push }, handler, onResolve, onReject, onCancel) => (immediatelyCancelError) => {
    if (immediatelyCancelError) {
        return onCancel()(immediatelyCancelError);
    }
    let cancel;
    let result;
    try {
        result = handler(params, abort => (cancel = abort));
    }
    catch (error) {
        return onReject()(error);
    }
    if (Object(result) === result && typeof result.then === 'function') {
        const promise = cancellable(result, cancel);
        push(promise);
        return promise.then(onResolve(promise), error => error instanceof CancelledError
            ? onCancel(promise)(error)
            : onReject(promise)(error));
    }
    return onResolve()(result);
};
const fin = ({ params, unpush, strategy, feedback, cancelAll }, target, field, fn) => (promise) => (data) => {
    const payload = [];
    if (!unpush(promise) || strategy === RACE || target.length === 1) {
        const body = { params, [field]: data };
        if (feedback) {
            body.strategy = strategy;
        }
        let i = target.length;
        while (i--) {
            payload.push(body);
        }
        if (strategy === RACE && target.length > 1) {
            cancelAll(strategy);
        }
    }
    else {
        target = [];
    }
    target.push(sidechain);
    payload.push([fn, data]);
    launch({
        target,
        params: payload,
        defer: true,
    });
};
const sidechain = createNode({
    node: [
        step.run({
            fn([fn, value]) {
                fn(value);
            },
        }),
    ],
    meta: { op: 'fx', fx: 'sidechain' },
});

==> effector-reeffect-1.1.0.tgz: package/dist-src/runner.js <==
import { createNode, launch, step } from 'effector';
import { CancelledError, LimitExceededError } from './error';
import { QUEUE, RACE, TAKE_FIRST, TAKE_LAST } from './strategy';
import { cancellable } from './promise';
import { assign } from './tools';
export const patchRunner = (runner, scope) => {
    assign(runner.scope, scope, {
        anyway: runner.scope.anyway.prepend(payload => assign({ status: payload.result ? 'done' : 'fail' }, payload)),
    });
    runner.meta.onCopy.push('cancelled', 'cancel');
    runner.seq = seq();
    scope.cancel.watch(() => scope.cancelAll());
};
const seq = () => [
    step.run({
        fn({ params, args, req }, { getHandler, done, fail, anyway, cancelled, strategy, feedback, limit, running, inFlight, push, unpush, cancelAll, }) {
            strategy = (args && args.strategy) || strategy;
            const scope = {
                params,
                strategy,
                feedback,
                push,
                unpush,
                cancelAll,
                inFlight,
            };
            const go = run(scope, getHandler(), fin(scope, [anyway, done], 'result', req.rs), fin(scope, [anyway, fail], 'error', req.rj), fin(scope, [cancelled], 'error', req.rj));
            if (running.length >= limit) {
                return go(new LimitExceededError(limit, running.length));
            }
            if (strategy === TAKE_FIRST && running.length > 0) {
                return go(new CancelledError(strategy));
            }
            if (strategy === TAKE_LAST && running.length > 0) {
                cancelAll(strategy);
            }
            if (strategy === QUEUE && running.length > 0) {
                const promise = cancellable(Promise.all(running.map(p => p.catch(() => { }))));
                push(promise);
                return promise.then(() => (unpush(promise), go()), error => (unpush(promise), go(error)));
            }
            go();
        },
    }),
];
const run = ({ params, push }, handler, onResolve, onReject, onCancel) => (immediatelyCancelError) => {
    if (immediatelyCancelError) {
        return onCancel()(immediatelyCancelError);
    }
    let cancel;
    let result;
    try {
        result = handler(params, abort => (cancel = abort));
    }
    catch (error) {
        return onReject()(error);
    }
    if (Object(result) === result && typeof result.then === 'function') {
        const promise = cancellable(result, cancel);
        push(promise);
        return promise.then(onResolve(promise), error => error instanceof CancelledError
            ? onCancel(promise)(error)
            : onReject(promise)(error));
    }
    return onResolve()(result);
};
const fin = ({ params, unpush, strategy, feedback, cancelAll, inFlight }, target, field, fn) => (promise) => (data) => {
    const payload = [];
    const runningCount = unpush(promise);
    if (!runningCount || strategy === RACE || target.length === 1) {
        const body = { params, [field]: data };
        if (feedback) {
            body.strategy = strategy;
        }
        let i = target.length;
        while (i--) {
            payload.push(body);
        }
        if (strategy === RACE && target.length > 1) {
            cancelAll(strategy);
        }
    }
    else {
        target = [];
    }
    target.push(inFlight, sidechain);
    payload.push(runningCount, [fn, data]);
    launch({
        target,
        params: payload,
        defer: true,
    });
};
const sidechain = createNode({
    node: [
        step.run({
            fn([fn, value]) {
                fn(value);
            },
        }),
    ],
    meta: { op: 'fx', fx: 'sidechain' },
});
